(function (window, document, rJS, RSVP) {

  function clearList(my_dom_node) {
    if (my_dom_node) {
      my_dom_node.parentNode.removeChild(my_dom_node);
    }
  }

  rJS(window)
  
    // ready = all dependencies loaded
    // make the gadget DOM element accessible throughout the script
    .ready(function (gadget) {
      return gadget.getElement()
        .push(function (element) {
          gadget.element = element;
        });
    })
    
    // initialize a jIO storage (sessionstorage in this case)
    .ready(function (gadget) {
      return new RSVP.Queue()
        .push(function () {
          return gadget.getDeclaredGadget('jio_gadget');
        })
        .push(function (my_gadget) {
          my_gadget.createJio({
            type: 'local',
            sessiononly: true
          });
        });
    })
    
    .declareMethod('get', function (param_list) {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.getDeclaredGadget('jio_gadget');
        })
        .push(function (my_gadget) {
          return my_gadget.get.apply(my_gadget, param_list);
        });
    })
    
    .declareMethod('getAttachment', function (param_list) {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.getDeclaredGadget('jio_gadget');
        })
        .push(function (my_gadget) {
          return my_gadget.getAttachment.apply(my_gadget, param_list);
        });
    })
    
    .declareMethod('putAttachment', function (param_list) {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.getDeclaredGadget('jio_gadget');
        })
        .push(function (my_gadget) {
          return my_gadget.putAttachment.apply(my_gadget, param_list);
        });
    })
    
    .declareMethod('removeAttachment', function (param_list) {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.getDeclaredGadget('jio_gadget');
        })
        .push(function (my_gadget) {
          return my_gadget.removeAttachment.apply(my_gadget, param_list);
        });
    })
    
    .declareMethod('allAttachments', function (param_list) {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.getDeclaredGadget('jio_gadget');
        })
        .push(function (my_gadget) {
          return my_gadget.allAttachments.apply(my_gadget, param_list);
        });
    })
    
    // output all attachments
    .declareMethod('listDocument', function () {
      var gadget = this;
      clearList(gadget.element.querySelector(".list"));
      return new RSVP.Queue()
        .push(function () {
          return gadget.allAttachments(['/']);
        })
        .push(function (my_result) {
          var promise_list = [];
          for (var key in my_result) {
            promise_list.push(gadget.getAttachment(['/', key]));
          }
          return RSVP.all(promise_list);
        })
        .push(function (my_result_list) {
          var promise_list = [],
            i = my_result_list.length;
          while (i--) {
            // blob reading is provided through gadget_global
            promise_list.push(promiseReadAsText(my_result_list[i]));
          }
          return RSVP.all(promise_list);
        })
        .push(function (my_result_list) {
          var list = document.createElement("ul"),
            i = my_result_list.length;
          while (i--) {
            var result = document.createElement("li");
            result.textContent = my_result_list[i];
            list.appendChild(result);
          }
          list.className = "list";
          gadget.element.appendChild(list);
        });
    })
    
    // remove all attachments stored in localStorage
    .declareMethod("clearDocumentList", function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.allAttachments(['/']);
        })
        .push(function (my_result_dict) {
          var promise_list = [],
            key;
            
          for(key in my_result_dict) {
            if(my_result_dict.hasOwnProperty(key)) {
              promise_list.push(gadget.removeAttachment(['/', key]));
            }
          }

          return RSVP.all(promise_list);
        })
        .push(function () {
          clearList(gadget.element.querySelector(".list"));
        });
    })
    
    // example: form submit, uses declareService, which fires when DOM is built!
    .declareService(function () {
      var gadget = this,
        form = gadget.element.querySelector("form");
      function callback(my_event) {
        my_event.preventDefault();
        return new RSVP.Queue()
          .push(function () {
            return gadget.clearDocumentList();
          })
          .push(function () {
            return false;
          });
      }
      
      // event binding is provided through gadget_global
      return loopEventListener(form, "submit", false, callback);
    })

    // example: event binding
    .declareService(function () {
      var gadget = this;
      function notifyRefresh() {
        return gadget.listDocument();
      }
      return loopEventListener(
        gadget.element.querySelector('.refresh'),
        'click',
        false,
        notifyRefresh
      );
    });
    
}(window, document, rJS, RSVP));